import os


def split_window_evenly(splitnum: int, vertical: bool = True) -> int:
    tmux_cmd = "tmux "
    for idx, i in enumerate(reversed(range(splitnum))):
        if i == 0:
            break
        tmux_cmd += "split-window "
        tmux_cmd += f"{'-v' if vertical else '-h'} "
        tmux_cmd += f"-p {100 - int(100 * i / (splitnum - idx))} "
        tmux_cmd += "-d \\; "
    return os.system(tmux_cmd)


def send_keys(cmd: str, enter: bool = True) -> int:
    tmux_cmd = f"tmux send-keys '{cmd}' {'Enter' if enter else ''}"
    return os.system(tmux_cmd)


def select_pane(direction: str, ntimes: int = 1) -> int:
    assert direction in ["U", "D", "R", "L"]

    tmux_cmd = "tmux "
    for _ in range(ntimes):
        tmux_cmd += f"select-pane -{direction}\\; "
    return os.system(tmux_cmd)


def new_window(name: str = "") -> int:
    tmux_cmd = "tmux new-window"
    if name:
        tmux_cmd += f" -n {name}"
    return os.system(tmux_cmd)



import sys
from argparse import Namespace

from arguments.TargsArgumentParser import TargsArgumentParser


class Parser:
    @staticmethod
    def parse_args() -> Namespace:
        # Configure parser
        parser = TargsArgumentParser(add_help=False, allow_abbrev=False)
        parser.add_argument("--help", action="store_true", help="show this help message and exit")
        parser.add_argument("-h", dest="horizontal", metavar="N", type=int, default=2, help="# of windows in horizontal")
        parser.add_argument("-v", dest="vertical", metavar="N", type=int, default=2, help="# of windows in vertical")
        parser.add_argument("-I", metavar="replstr", dest="replstr", type=str)

        # Modify usage (add " [-- utility [argument ...]]")
        """
        e.g.
        "usage: args.py [--help] [-h N] [-v N] [-I replstr]\n"
        ->
        "args.py [--help] [-h N] [-v N] [-I replstr] [-- utility [argument ...]]\n"
        """
        usage = parser.format_usage().strip()[8:]
        parser.usage = usage + " [-- utility [argument ...]]\n"

        args: Namespace = parser.parse_args()
        if args.help:
            parser.print_help()
            exit()
        return args

    @staticmethod
    def parse_stdin(delimiter="\n") -> list[str]:
        # Read stdin, skipping blank lines
        inputs: list[str] = sys.stdin.read().split(delimiter)
        inputs = [cmd for cmd in inputs if cmd != ""]

        if len(inputs) == 0:
            exit()

        return inputs



import argparse
import sys
from argparse import Namespace


class TargsArgumentParser(argparse.ArgumentParser):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def parse_args(self, args=None, namespace=None):

        argv = args if args else sys.argv[1:]
        has_util: bool = True
        try:
            sep_idx = argv.index("--")
            if len(argv) == sep_idx + 1:
                has_util = False
                argv = argv[:sep_idx]
        except ValueError:
            has_util = False

        # Parse arguments
        ret_args: Namespace
        if has_util:
            ret_args = super().parse_args(argv[:sep_idx], namespace=namespace)
            setattr(ret_args, "cmd", argv[sep_idx + 1:])
        else:
            ret_args = super().parse_args(argv, namespace=namespace)
            setattr(ret_args, "cmd", ["echo", "{}"])
            setattr(ret_args, "replstr", "{}")

        return ret_args



from setuptools import setup, find_packages

DESCRIPTION = "targs: tmux + xargs"
NAME = "tmux_xargs"
AUTHOR = "Yusuke Uchida"
AUTHOR_EMAIL = "fantomyuu0623@gmail.com"
URL = "https://github.com/ucchiee/targs"
LICENSE = "MIT"
DOWNLOAD_URL = URL
VERSION = "0.0.8"
PYTHON_REQUIRES = ">=3.9"
KEYWORDS = "tmux xargs targs"
with open("README.md", "r", encoding="utf-8") as fp:
    readme = fp.read()
LONG_DESCRIPTION = readme
LONG_DESCRIPTION_CONTENT_TYPE = "text/markdown"
PY_MODULES = ["targs"]
ENTRY_POINTS = {"console_scripts": ["targs = targs:main"]}

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=AUTHOR,
    maintainer_email=AUTHOR_EMAIL,
    url=URL,
    download_url=URL,
    license=LICENSE,
    keywords=KEYWORDS,
    install_requires=[],
    packages=find_packages(),
    py_modules=PY_MODULES,
    entry_points=ENTRY_POINTS,
)



from argparse import Namespace
from math import ceil

import tmux
from arguments import Parser


def split_and_execute(args: Namespace, cmds: list[str]):
    # Split widnow vertically first.
    tmux.split_window_evenly(args.vertical, vertical=True)

    # Iterate over panes newly created above
    cmd_idx: int = 0
    for _ in range(args.vertical):

        # Split window horizontally
        tmux.split_window_evenly(args.horizontal, vertical=False)

        for _ in range(args.horizontal):
            if len(cmds) == cmd_idx:
                # # of commands can be less than # of pane
                break

            # Execute a command and move right
            tmux.send_keys(cmds[cmd_idx], enter=True)
            tmux.select_pane("R", ntimes=1)
            cmd_idx += 1

        tmux.select_pane("D", ntimes=1)


def main():

    args: Namespace = Parser.parse_args()

    inputs: list[str] = Parser.parse_stdin(delimiter="\n")

    """
    e.g.
    ["ping", "{}"] -> "ping {}"
    """
    cmd = " ".join(args.cmd)

    """
    e.g.
    "ping {}" -> ["ping google.com", "ping github.com"]
    """
    cmds = [cmd.replace(args.replstr, line) if args.replstr else cmd for line in inputs]

    # Iterate over windows
    num_panes: int = args.vertical * args.horizontal
    num_windows: int = ceil(len(cmds) / num_panes)
    for i in range(num_windows):
        tmux.new_window(f"targs{i}")
        split_and_execute(args, cmds[num_panes * i: num_panes * (i + 1)])


if __name__ == "__main__":
    main()
